package it.unicam.cs.asdl1819.bankaccounts;

import java.util.Comparator;

/**
 * 
 * Conto corrente di base, senza nessuna particolare caratteristica.
 * 
 * @author luca
 *
 */
public abstract class BankAccount implements Comparable<BankAccount> {

    /**
     * Comparator per un ordinamento diverso da quello naturale, che ordina due
     * conti in base al nome dell'intestatario. In caso di nome uguale l'ordine
     * è dato dall'iban.
     */
    public static final Comparator<BankAccount> ORDINA_PER_NOME = new Comparator<BankAccount>() {
        public int compare(BankAccount b1, BankAccount b2) {
            int cmp = b1.nome.compareTo(b2.nome);
            // Controllo per la compatibilità con equals
            if (cmp != 0)
                return cmp;
            // se i due conti correnti hanno lo stesso intestatario, faccio la
            // comparazione con l'ordinamento naturale tra gli iban
            return b1.compareTo(b2);
        }
    };

    /**
     * Comparator per un ordinamento diverso da quello naturale, che ordina due
     * conti in base al saldo del conto. In caso di saldo uguale l'ordine è dato
     * dall'iban.
     */
    public static final Comparator<BankAccount> ORDINA_PER_SALDO = new Comparator<BankAccount>() {
        public int compare(BankAccount b1, BankAccount b2) {
            Double saldo1 = new Double(b1.saldo);
            Double saldo2 = new Double(b2.saldo);
            int cmp = saldo1.compareTo(saldo2);
            // Controllo per la compatibilità con equals
            if (cmp != 0)
                return cmp;
            // se i due conti correnti hanno lo stesso saldo, faccio la
            // comparazione con l'ordinamento naturale tra gli iban
            return b1.compareTo(b2);
        }
    };

    private double saldo;

    private String nome;

    private final String iban;

    /**
     * @param saldo
     *                  saldo iniziale
     * @param nome
     *                  nome dell'intestatario
     * @param iban
     *                  codice unico identificativo del conto, immutabile
     */
    public BankAccount(double saldo, String nome, String iban) {
        this.saldo = saldo;
        this.nome = nome;
        this.iban = iban;
    }

    /**
     * @return the nome
     */
    public String getNome() {
        return nome;
    }

    /**
     * @param nome
     *                 the nome to set
     */
    public void setNome(String nome) {
        this.nome = nome;
    }

    /**
     * @return the saldo
     */
    public double getSaldo() {
        return saldo;
    }

    /**
     * @return the iban
     */
    public String getIban() {
        return iban;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((iban == null) ? 0 : iban.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * l'equals di conti correnti si basa sull'iban, identificatore unico
     * immutabile
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (!(obj instanceof BankAccount))
            return false;
        BankAccount other = (BankAccount) obj;
        if (iban == null) {
            if (other.iban != null)
                return false;
        } else if (!iban.equals(other.iban))
            return false;
        return true;
    }

    /**
     * Metodo astratto che viene chiamato sul conto alla fine di ogni mese.
     * Esegue azioni specifiche del conto da fare alla fine del mese. Dipende
     * dalle varie sottoclassi.
     * 
     */
    public abstract void endOfMonth();

    /**
     * Deposita un importo nel conto.
     * 
     * @param importo
     *                    l'ammontare da depositare
     */
    public void deposit(double importo) {
        if (importo < 0)
            throw new IllegalArgumentException(
                    "Tentativo di deposito di importo negativo");
        saldo += importo;
    }

    /**
     * Preleva un importo dal conto.
     * 
     * @param importo
     *                    l'ammontare da prelevare
     */
    public void withdraw(double importo) {
        if (importo < 0)
            throw new IllegalArgumentException(
                    "Tentativo di prelievo di importo negativo");
        if (importo > saldo)
            throw new InsufficientFundException(
                    "Tentativo di prelevare più del posseduto");
        saldo -= importo;

    }

    /**
     * Ordinamento tra conti correnti tramite l'ordinamento delle stringhe iban.
     * 
     * @param other
     *                  il conto corrente da comparare con questo
     * @return un valore < 0 se questo conto corrente precede other
     *         nell'ordinamento, 0 se i due conti correnti coincidono, un valore
     *         > 0 se questo conto corrente segue other nell'ordinamento.
     */
    public int compareTo(BankAccount other) {
        return this.iban.compareTo(other.iban);
    }

}
